/*********************************************************************

   Copyright (C) 2001 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

   --------

   This code is based, in part, on:

      "Syntax Coloring Text Edit Window Class " - Randy More (9, May 1998)
      http://www.codeguru.com/editctrl/syntax_coloring.shtml

   Though probably only 5% of the original code remains.

**********************************************************************/
#ifndef _COLORIZE_
#define _COLORIZE_

#pragma warning(disable:4786)
#include <vector>
#include <set>
using namespace std;
#pragma warning(disable:4786)

#define KEYWORD_COLOR               RGB(0,0,1)//RJA making keyword color almost black RGB(0,0,160)
#define PASM_KEYWORD_COLOR          RGB(128,0,0)//RJA making keyword color almost black RGB(0,0,160)
#define FLOW_COLOR					RGB(128,128,128)
#define NORMAL_COLOR                RGB(0,0,0)
#define POUND_COLOR					RGB(0,0,128)

/*
   The base class only provides keyword coloring support.

   In this class, a "word" is a sequence of uninterrupted alpha-numeric
   characters. The keywords are loaded with the LoadKeywordFile member.
   One keyword per line in a plain text file.

   You can do more-complex processing here, or in a derived class.

*/

class CColorizer
{
public:
   CColorizer();

	virtual ~CColorizer();

   // called once per text line, when the line colors need updating.
   // 'colors' will be filled with one COLORREF per text character.
   //
   // you will definitely rewrite this function (in a derived class, probably) for your own application.
   // 
	virtual void SetLineColors(CStringA &pString, vector<COLORREF>& colors, COLORREF LineBackgroundColor, int nBraces); //RJA removed const, added LineBackgroundColor and brace count
	
   // load magic keywords.
   virtual void LoadKeywordFile(CString &keywordsFile);
   //load operators
   void CColorizer::LoadOperators(); //RJA
   void CColorizer::LoadFlowControl();//rja

	BOOL m_bCompareCase;

protected:
	virtual BOOL IsPasm1KeyWord(CStringA & pTest);
	virtual BOOL CColorizer::IsSpin1KeyWord(CStringA & pTest);
	virtual BOOL CColorizer::IsPasm1Reg(CStringA & test);
	virtual BOOL IsPasm2KeyWord(CStringA & pTest);
	virtual BOOL CColorizer::IsSpin2KeyWord(CStringA & pTest);
	virtual BOOL CColorizer::IsPasm2Reg(CStringA & test);
	virtual BOOL CColorizer::IsOperator(CStringA & pTest);
	virtual BOOL CColorizer::IsFlowControl(CStringA & test);
	virtual BOOL CColorizer::IsVarConKeyWord(CStringA & test);
	virtual BOOL CColorizer::IsObjKeyWord(CStringA & test);


   
   // set of keywords
   std::set<CStringA> m_stringSet;
   std::set<CStringA> m_stringSetVarCon;
   std::set<CStringA> m_stringSetObj;
   std::set<CStringA> m_stringSet2; //P2
   std::set<CStringA> m_stringSetPASM;
   std::set<CStringA> m_stringSetP2ASM;
   std::set<CStringA> m_stringSetP1Regs;
   std::set<CStringA> m_stringSetP2Regs;
   std::set<CStringA> m_Operators;
   std::set<CStringA> m_FlowControl;

   bool CColorizer::isalnumSpin(int c);

   CString m_sColorMode;

};

#endif