﻿/*********************************************************
Copyright (C) 2018 Raymond Allen

MIT License:

┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
**********************************************************************/
/*********************************************************************

   Copyright (C) 2001 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

   --------

   This code is based, in part, on:

      "Syntax Coloring Text Edit Window Class " - Randy More (9, May 1998)
      http://www.codeguru.com/editctrl/syntax_coloring.shtml


   Though probably only 5% of the original code remains.

**********************************************************************/

// SampleColorizer.cpp : implementation of the CSampleColorizer class
//

#include "stdafx.h"
#include "SampleColorizer.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////

/*
   SetLineColors

   This is where you decide how to color the text.
   
   The colors vector will have an element for each character in the input string.
   Each element in the vector is a COLORREF that tells the edit control which color to 
   use when drawing the corresponding character.

   It is expected that you will rewrite most of the code in this function to fit 
   your own application.

   This function, as written, has access to only a single line at a time. If you need
   to access the entire text buffer, you will have to find your own way to do that.


   
   This sample uses a keywords.ini to define a few keywords (set by the parent of the edit control). 
   It also does some context-sensitive keyword coloring in this function. 
   
   The rules for this CSampleColorizer are:

   symbol      rule
   ----------------
   # text      any line that starts with "#" is a comment
   @+          must be the only text on the line
   @-          must be the only text on the line
   @:text      must be at the start of the line
   
   There is nothing special about these rules. They simply demonstrate some simple ways
   of doing context-sensitive coloring.
   
*/

// demo colors 
#define COMMENT_COLOR               RGB(106,99,51)//RJA RGB(0,120,0)
#define SECTION_DECLARATION_COLOR   RGB(160,40,0)
#define ERROR_COLOR                 RGB(255,0,0)

COLORREF CSampleColorizer::CommentColor(COLORREF LineBackgroundColor, int nCommentCount)
{
	//PUB
	switch (LineBackgroundColor)
	{
		//pub
	case RGB(191, 223, 255):
		return (nCommentCount < 2) ? RGB(55, 111, 167) : RGB(50,78,105);
	case RGB(167, 210, 253):
		return (nCommentCount < 2) ? RGB(55, 105, 155) : RGB(49, 73, 98);;
		//PRI
	case RGB(191, 248, 255):
		return (nCommentCount < 2) ? RGB(55, 155, 167) : RGB(50, 99, 105);;
	case RGB(167, 243, 253):
		return (nCommentCount < 2) ? RGB(55, 144, 155) : RGB(49, 92, 98);;
		//CON
	case RGB(253, 243, 168):
		return (nCommentCount < 2) ? RGB(168, 155, 56) : RGB(106, 99, 51);;
	case RGB(255, 248, 192):
		return (nCommentCount < 2) ? RGB(156, 144, 55) : RGB(98, 92, 49);;
		//VAR
	case RGB(255, 223, 191):
		return (nCommentCount < 2) ? RGB(167, 111, 55) : RGB(105, 78, 50);;
	case RGB(253, 210, 167):
		return (nCommentCount < 2) ? RGB(155, 105, 55) : RGB(98, 73, 49);;
		//DAT
	case RGB(191, 255, 200):
		return (nCommentCount < 2) ? RGB(55, 167, 71) : RGB(50, 105, 50);;
	case RGB(167, 253, 179):
		return (nCommentCount < 2) ? RGB(55, 155, 69) : RGB(49, 98, 55);;
		//OBJ
	case RGB(255, 191, 191):
		return (nCommentCount < 2) ? RGB(167, 55, 55) : RGB(105, 50, 50);;
	case RGB(253, 167, 167):
		return (nCommentCount < 2) ? RGB(155, 55, 55) : RGB(98, 49, 49);;
	}
	return RGB(0,0,0);
}



void CSampleColorizer::SetLineColors(CStringA &csString, vector<COLORREF>& colors, COLORREF LineBackgroundColor, int nBraces)
{
   // nothing to do
   if (csString.GetLength() ==0) 
   {
      colors.clear();
      return;
   }


   // one COLORREF per input char
   colors.resize(csString.GetLength());

   /*/ all lines starting with a '#' are comment lines
   if ((csString[0]=='#'))
	{
		//set the line to all comment color
		for(int i=0; i<csString.GetLength(); i++)
		{
			colors.at(i) = COMMENT_COLOR;
		}
	}
	


	*/

   {
	   // let the base class look for keywords
	   CColorizer::SetLineColors(csString, colors, LineBackgroundColor, nBraces);
   }




   //Things between quotes are in red
   int j = csString.Find("\"", 0);
   while (j >= 0)
   {
	   int k= csString.Find("\"", j+1);
	   if (k<0)
		   k= csString.GetLength()-1;
	   //everything after the " to red if count is odd

		for (int i = j; i <=k;  i++)
		{
			colors.at(i) = RGB(247,51,51);//rja quote color
		}
	   j = csString.Find("\"", ++k);
   }
  

   // Everything after a ''' is a comment
   //need dark version of background color
   COLORREF darkBackground = CommentColor(LineBackgroundColor,1);// RGB(GetRValue(LineBackgroundColor) * 5 / 8, GetBValue(LineBackgroundColor) * 5 / 8, GetGValue(LineBackgroundColor) * 5 / 8);
   COLORREF darkerBackground = CommentColor(LineBackgroundColor, 2);

   int nCount = 0;
   j = csString.Find("'", 0);
   while (j>=0)
   {
	   nCount++;
	   //everything after the ' to comment color
	   for (int i=j; i<csString.GetLength(); i++)
	   {
		   colors.at(i) = (nCount>1) ? darkerBackground : darkBackground;//rja COMMENT_COLOR;
	   }
	   j = csString.Find("'", ++j);
   }

   //Handle braces
   //everything with brace count >0 gets comment color
   for (int i = 0; i < csString.GetLength(); i++)
   {
	  if (nBraces>0)
		colors.at(i) = (nBraces > 1) ? darkerBackground : darkBackground;//rja COMMENT_COLOR;
	  if (csString[i] == '}')
		  nBraces--;
	  if (csString[i] == '{')
	  {
		  nBraces++;
		  colors.at(i) = (nBraces > 1) ? darkerBackground : darkBackground;//rja COMMENT_COLOR;
	  }
   }

   //color all numbers that are not braced or commented
   CStringA s = csString;
   j = s.FindOneOf("0123456789");
   int k = j;
   while (j >= 0)
   {
	   COLORREF c = colors.at(k);
	   if (c== NORMAL_COLOR)
		colors.at(k) = RGB(0, 128, 128);//rja number color
	   s = s.Right(s.GetLength() - j - 1);
	   k++;
	   j = s.FindOneOf("0123456789");
	   k += j;
   };
  


}
